@@ -37,7 +37,8 @@ class QgsLayerTree;
 
 /**
  * \ingroup core
- * The QgsLayerTreeModel class is model implementation for Qt item views framework.
+ * \brief The QgsLayerTreeModel class is model implementation for Qt item views framework.
+ *
  * The model can be used in any QTreeView, it is however recommended to use it
  * with QgsLayerTreeView which brings additional functionality specific to layer tree handling.
  *
@@ -314,6 +315,12 @@ class CORE_EXPORT QgsLayerTreeModel : public QAbstractItemModel
     void nodeLayerWillBeUnloaded();
     void layerLegendChanged();
 
+    /**
+     * Emitted when layer flags have changed.
+     * \since QGIS 3.18
+     */
+    void layerFlagsChanged();
+
     void layerNeedsUpdate();
 
     void legendNodeDataChanged();
@@ -340,7 +347,7 @@ class CORE_EXPORT QgsLayerTreeModel : public QAbstractItemModel
      * Emits dataChanged() for all scale dependent layers.
      * \since QGIS 2.16
      */
-    void refreshScaleBasedLayers( const QModelIndex &index = QModelIndex() );
+    void refreshScaleBasedLayers( const QModelIndex &index = QModelIndex(), double previousScale = 0.0 );
 
     static QIcon iconGroup();
 
@@ -435,6 +442,12 @@ class CORE_EXPORT QgsLayerTreeModel : public QAbstractItemModel
     //! Per layer data about layer's legend nodes
     QHash<QgsLayerTreeLayer *, LayerLegendData> mLegend;
 
+    /**
+     * Keep track of layer nodes for which the legend
+     * size needs to be recalculated
+     */
+    QSet<QgsLayerTreeLayer *> mInvalidatedNodes;
+
     QFont mFontLayer;
     QFont mFontGroup;
 
@@ -452,10 +465,14 @@ class CORE_EXPORT QgsLayerTreeModel : public QAbstractItemModel
     double mLegendMapViewScale;
     QTimer mDeferLegendInvalidationTimer;
 
+  private slots:
+    void legendNodeSizeChanged();
+
   private:
 
     //! Returns a temporary render context
     QgsRenderContext *createTemporaryRenderContext() const;
+
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsLayerTreeModel::Flags )
@@ -485,6 +502,8 @@ class EmbeddedWidgetLegendNode : public QgsLayerTreeModelLegendNode
     {
       if ( role == RuleKeyRole )
         return mRuleKey;
+      else if ( role == QgsLayerTreeModelLegendNode::NodeTypeRole )
+        return QgsLayerTreeModelLegendNode::EmbeddedWidget;
       return QVariant();
     }
 
